model TemperatureStateEstimation "Analyze robustness of a state estimator"
  inner Modelica.Blocks.Noise.GlobalSeed globalSeed annotation(Placement(visible = true, transformation(origin = {-160, 120}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Noise.NormalNoise temperatureMeasurementNoise(sigma = 10) annotation(Placement(visible = true, transformation(origin = {-40, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Noise.NormalNoise voltageControlNoise(sigma = 0.25, mu = 10) annotation(Placement(visible = true, transformation(origin = {-160, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant voltageControlSignal(k = 24) annotation(Placement(visible = true, transformation(origin = {-160, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Sum voltageControlInput(nin = 2) annotation(Placement(visible = true, transformation(origin = {-120, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Sum temperatureMeasurement(nin = 2) annotation(Placement(visible = true, transformation(origin = {0, 80}, extent = {{10, -10}, {-10, 10}}, rotation = -90)));
  Modelica.Blocks.Sources.Ramp ambientTemperatureSignal(duration = 24, height = 15, offset = 295) annotation(Placement(visible = true, transformation(origin = {150, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  Modelica.Blocks.Noise.NormalNoise ambientTemperatureMeasurementNoise(sigma = 0.5) annotation(Placement(visible = true, transformation(origin = {80, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Sum ambientTemperatureMeasurement(nin = 2) annotation(Placement(visible = true, transformation(origin = {120, 80}, extent = {{10, -10}, {-10, 10}}, rotation = -90)));
  Modelica.Blocks.Continuous.StateSpace stateEstimator(y_start = {295}, initType = Modelica.Blocks.Types.Init.InitialOutput, A = {{-0.2562858599079851}}, B = {{0.006285860555044676, 0.24998750062490563, 0.04799880110982621}}, C = {{1}}, D = {{0, 0, 0}}) annotation(Placement(visible = true, transformation(origin = {-0, 130}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 100, T(fixed = true)) annotation(Placement(transformation(extent = {{0, -60}, {20, -80}}, origin = {-10, 50}, rotation = 0), visible = true));
  Modelica.Electrical.Analog.Basic.Resistor heatingResistor(R = 10, useHeatPort = true) annotation(Placement(transformation(origin = {-40, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270), visible = true));
  Modelica.Thermal.HeatTransfer.Sensors.TemperatureSensor temperatureSensor annotation(Placement(transformation(origin = {0, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 25) annotation(Placement(transformation(extent = {{40, -60}, {60, -40}}, origin = {-10, 50}, rotation = 0), visible = true));
  Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{-100, -100}, {-80, -80}}, origin = {10, 40}, rotation = 0), visible = true));
  Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(Placement(visible = true, transformation(origin = {-80, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
  Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature ambientTemperature annotation(Placement(visible = true, transformation(origin = {80, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
equation
  connect(voltageControlSignal.y, voltageControlInput.u[1]) annotation(Line(visible = true, origin = {-142.75, 40}, points = {{-6.25, 40}, {2.75, 40}, {2.75, -40}, {10.75, -40}}, color = {1, 37, 163}));
  connect(voltageControlNoise.y, voltageControlInput.u[2]) annotation(Line(visible = true, origin = {-140.25, -40}, points = {{-8.75, 40}, {0.25, 40}, {0.25, 40}, {8.25, 40}}, color = {1, 37, 163}));
  connect(voltageControlInput.y, signalVoltage.v) annotation(Line(visible = true, origin = {-98, 0}, points = {{-11, 0}, {11, -0}}, color = {1, 37, 163}));
  connect(temperatureMeasurementNoise.y, temperatureMeasurement.u[2]) annotation(Line(visible = true, origin = {-9.667, 62.667}, points = {{-19.333, -2.667}, {9.667, -2.667}, {9.667, 5.333}}, color = {1, 37, 163}));
  connect(temperatureSensor.T, temperatureMeasurement.u[1]) annotation(Line(visible = true, origin = {0, 49}, points = {{0, -19}, {0, 19}}, color = {1, 37, 163}));
  connect(ambientTemperatureSignal.y, ambientTemperature.T) annotation(Line(visible = true, origin = {115.5, 0}, points = {{23.5, 0}, {-23.5, -0}}, color = {1, 37, 163}));
  connect(ambientTemperatureMeasurementNoise.y, ambientTemperatureMeasurement.u[2]) annotation(Line(visible = true, origin = {110.333, 62.667}, points = {{-19.333, -2.667}, {9.667, -2.667}, {9.667, 5.333}}, color = {1, 37, 163}));
  connect(ambientTemperature.T, ambientTemperatureMeasurement.u[1]) annotation(Line(visible = true, origin = {110.667, 22.667}, points = {{-18.667, -22.667}, {9.333, -22.667}, {9.333, 45.333}}, color = {1, 37, 163}));
  connect(temperatureMeasurement.y, stateEstimator.u[1]) annotation(Line(visible = true, origin = {-0, 104.5}, points = {{0, -13.5}, {-0, 13.5}}, color = {1, 37, 163}));
  connect(ambientTemperatureMeasurement.y, stateEstimator.u[2]) annotation(Line(visible = true, origin = {60, 119.303}, points = {{60, -28.303}, {60, -19.303}, {-60, -19.303}, {-60, -1.303}}, color = {1, 37, 163}));
  connect(voltageControlSignal.y, stateEstimator.u[3]) annotation(Line(visible = true, origin = {-85.8, 109.165}, points = {{-63.2, -29.165}, {-54.2, -29.165}, {-54.2, -9.165}, {85.8, -9.165}, {85.8, 8.835}}, color = {1, 37, 163}));
  connect(heatingResistor.heatPort, thermalConductor.port_a) annotation(Line(points = {{-20, -50}, {40, -50}}, color = {191, 0, 0}, visible = true, origin = {-10, 50}));
  connect(heatingResistor.heatPort, temperatureSensor.port) annotation(Line(points = {{-20, -50}, {10, -50}, {10, -40}}, color = {191, 0, 0}, visible = true, origin = {-10, 50}));
  connect(heatingResistor.heatPort, heatCapacitor.port) annotation(Line(points = {{-20, -50}, {10, -50}, {10, -60}}, color = {191, 0, 0}, visible = true, origin = {-10, 50}));
  connect(ground.p, signalVoltage.p) annotation(Line(visible = true, origin = {-80, -25}, points = {{0, -15}, {0, 15}}, color = {10, 90, 224}));
  connect(heatingResistor.n, signalVoltage.p) annotation(Line(visible = true, origin = {-55, -30}, points = {{15, 20}, {15, -10}, {-25, -10}, {-25, 20}}, color = {10, 90, 224}));
  connect(ambientTemperature.port, thermalConductor.port_b) annotation(Line(visible = true, origin = {60, 0}, points = {{10, 0}, {-10, -0}}, color = {191, 0, 0}));
  connect(signalVoltage.n, heatingResistor.p) annotation(Line(visible = true, origin = {-60, 35}, points = {{-20, -25}, {-20, 5}, {20, 5}, {20, -25}}, color = {10, 90, 224}));
  annotation(preferredView = "info", experiment(StopTime = 24), Diagram(coordinateSystem(extent = {{-180, -60}, {170, 150}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), uses(Modelica(version = "4.0.0")), __Wolfram(openClass = TemperatureStateEstimation, PlotSet(plots = {Plot(name = "Temperature Estimation", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = stateEstimator.y[1], legend = "Estimated Temperature"), Curve(x = time, y = temperatureMeasurement.y, legend = "Measured Temperature"), Curve(x = time, y = temperatureSensor.T, legend = "Actual Temperature")}), SubPlot(curves = {Curve(x = time, y = stateEstimator.y[1], legend = "Estimated Temperature"), Curve(x = time, y = temperatureSensor.T, legend = "Actual Temperature")})}), Plot(name = "Ambient Temperature Measurement", subPlots = {SubPlot(curves = {Curve(x = time, y = ambientTemperatureMeasurement.y, legend = "Measured Temperature"), Curve(x = time, y = ambientTemperature.T, legend = "Actual Temperature"), Curve(x = time, y = ambientTemperatureMeasurementNoise.y, legend = "Signal Noise")})}), Plot(name = "Temperature Measurement", subPlots = {SubPlot(curves = {Curve(x = time, y = temperatureMeasurement.y, legend = "Measured Temperature"), Curve(x = time, y = temperatureMeasurementNoise.y, legend = "Signal Noise"), Curve(x = time, y = temperatureSensor.T, legend = "Actual Temperature")})}), Plot(name = "Voltage Signal", subPlots = {SubPlot(curves = {Curve(x = time, y = voltageControlInput.y, legend = "Actual Voltage"), Curve(x = time, y = voltageControlSignal.y, legend = "Set Voltage"), Curve(x = time, y = voltageControlNoise.y, legend = "Signal Noise")})})})), Documentation(info = "<html>
<p>This model analyzes the robustness of a state estimator, taking measurements from an electrical radiator. The state estimator blends together different measurements and a model of the system in order to estimate the current radiator temperature.</p>

<p>The model contains a small electric radiator that is powered by a 24V voltage source. The radiator element heats up the body of the radiator. The temperature of the radiator will also be affected by the ambient temperature, which will increase during the simulation. Three measurements are taken continuously: radiator voltage, ambient temperature and a direct measurement of the radiator temperature. Using the Modelica.Noise library, a white noise is added to each of the measurements. All noisy measurements are fed into a Kalman filter that estimates the radiator temperature.&nbsp;</p>

<h4>Noise Distributions</h4>

<p>Expectation value and standard deviation for the noise distributions can be changed by highlighting the noise sources and changing the mu and sigma parameters, respectively. The Kalman filter was created assuming the standard values for the noise distributions in the model are true. By increasing the noise or any other parameter in the model, you can see how robust the filter is to modeling errors.</p>

<p style=\"text-align: center;\"><img src=\"modelica://TemperatureStateEstimation/./DocumentationFiles/parameters.png\" style=\"width:580px\" alt=\"\" /></p>

<h4>Simulation</h4>

<p>To simulate the model, click the Simulate button: <img src=\"modelica://TemperatureStateEstimation/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"\" /></p>

<h3>Plot the Results</h3>


<p>When first simulating the model, you will see the Temperature Estimation plots:</p>

<p style=\"text-align: center;\"><img src=\"modelica://TemperatureStateEstimation/./DocumentationFiles/tempplots.png\" style=\"width:600px\" alt=\"\" /></p>


<p>In the plot, the actual temperature is compared to the temperature estimated by the Kalman filter. In the top plot, the noisy measurements taken by the temperature sensor are also displayed.</p>

<p>The other available plots Ambient Temperature Measurement, Temperature Measurement and Voltage Signal focus on measurement, each showing the measured signal, the actual signal and the added noise.</p>

</html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, uses(Modelica(version = "4.0.0")), initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}));
end TemperatureStateEstimation;
